 /***********************************************
 *	FILE:	Accel.h
 *
 *	AUTHORS:	-Guillermo Maroto
 *	Based on Aeroquad.com
 *        
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/
#pragma once

#include "Sensor.h"

class Accel : public Sensor {
public:
  float accelScaleFactor;
  float smoothFactor;
  float rawAltitude;
  int accelChannel[3];
  int accelZero[3];
  float accelData[3];
  int accelADC[3];
  int sign[3];
  float accelOneG, zAxis;
  byte rollChannel, pitchChannel, zAxisChannel;
  
  Accel(void) {
    sign[ROLL] = -1;
    sign[PITCH] = 1;
    sign[YAW] = 1;
    zAxis = 0;
  }

  // ******************************************************************
  // The following function calls must be defined in any new subclasses
  // ******************************************************************
  virtual void initialize(void) {
    this->_initialize(rollChannel, pitchChannel, zAxisChannel);
  }
  virtual void measure(void);
  virtual void calibrate(void);
  virtual const int getFlightData(byte);

  // **************************************************************
  // The following functions are common between all Gyro subclasses
  // **************************************************************
  void _initialize(byte rollChannel, byte pitchChannel, byte zAxisChannel) {
    accelChannel[ROLL] = rollChannel;
    accelChannel[PITCH] = pitchChannel;
    accelChannel[ZAXIS] = zAxisChannel;
    accelOneG        = readFloat(ACCEL1G_ADR);
    accelZero[XAXIS] = readFloat(LEVELPITCHCAL_ADR);
    accelZero[YAXIS] = readFloat(LEVELROLLCAL_ADR);
    accelZero[ZAXIS] = readFloat(LEVELZCAL_ADR);
  }
  
  // return the raw ADC value from the accel, with sign change if need, not smoothed or scaled to SI units
  const int getRaw(byte axis) {
    return accelADC[axis] * sign[axis];
  }
  
  // return the smoothed and scaled to SI units value of the accel with sign change if needed
  const float getData(byte axis) {
    return accelData[axis] * sign[axis];
  }
  
  // invert the sign for a specifica accel axis
  const int invert(byte axis) {
    sign[axis] = -sign[axis];
    return sign[axis];
  }
  
  const int getZero(byte axis) {
    return accelZero[axis];
  }
  
  void setZero(byte axis, int value) {
    accelZero[axis] = value;
  }
  
  // returns the SI scale factor
  const float getScaleFactor(void) {
    return accelScaleFactor;
  }
  
  // returns the smoothfactor
  const float getSmoothFactor() {
    return smoothFactor;
  }
  
  void setSmoothFactor(float value) {
    smoothFactor = value;
  }
  
  void setOneG(float value) {
    accelOneG = value;
  }
  
  const float getOneG(void) {
    return accelOneG;
  }
  
  const float getZaxis() {
    return accelOneG - getData(ZAXIS);
  }
};
