/***********************************************
 *	FILE:	Quadcpter.h
 *
 *	AUTHORS:	-Guillermo Maroto
 *
 *           
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/
#pragma once

#include "MarotoCopter.h"
#include "State.h"
#include "PID.h"
#include "Motor_PWM.h"
#include "XBeeSerial.h"
#include "DataStorage.h"
#include "Receiver_AeroQuadMega.h"

// Tiempo de espera hasta el ARMADO inicial de los ESC
#define TIEMPOARMADOESC 3000  

class Quadcopter : public State 
{

private:
	Motor_PWM	motor[LASTMOTOR];
	PID			control[LASTAXIS];
	
	PID			rollAttitudeStableMode,
				pitchAttitudeStableMode; 
	
	PID			controlHeading;
	
	float		TargetPositionUser[LASTAXIS];
	int			Power;//Range 1000-2000
	
	XBeeSerial	comunication;
	byte queryType;
	
	Receiver_AeroQuadMega receiver;
	
	// Heading hold
	byte headingHoldConfig;
	//float headingScaleFactor;
	float commandedYaw;
	float headingHold; // calculated adjustment for quad to go to heading (PID output)
	float heading; // measured heading from yaw gyro (process variable)
	float relativeHeading; // current heading the quad is set to (set point)
	float setHeading;
	unsigned long headingTime;
	byte headingHoldState;
	
	unsigned long	currentTime;
	
public:
	
	Quadcopter() : State ()
	{
		Power=1000; 
		TargetPositionUser[PITCH]=0;
		TargetPositionUser[ROLL]=0;
		TargetPositionUser[YAW]=0;
		
		// Heading hold
		headingHoldConfig = 1;
		//float headingScaleFactor;
		commandedYaw = 0;
		headingHold = 0; // calculated adjustment for quad to go to heading (PID output)
		heading = 0; // measured heading from yaw gyro (process variable)
		relativeHeading = 0; // current heading the quad is set to (set point)
		setHeading = 0;
		headingTime = micros();
		headingHoldState = OFF;
	}
	
	void initialize();
	void processFlightControl();
	float processHeading();
	void initializeMotors();
	void initializeControl();
	void initializeComunication();
	void readComunication();
	void writeComunication();
	void zeroIntegralError();
	void armed();
	void disarmed();
	void initializeReceiver();
	void readReceiver();
	void setCurrentTime(unsigned long cTime){currentTime = cTime;}
	
};