/***********************************************
 *	FILE:	SerialCom.cpp
 *
 *	AUTHORS:	-Guillermo Maroto
 *
 *	Based on Aeroquad.com/SerialCom.pde
 *           
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/
#include "SerialCom.h"

// Serial functions

void PrintValueComma(float val) {
	Serial.print(val);
	comma();
}

void PrintValueComma(double val) {
	Serial.print(val);
	comma();
}

void PrintValueComma(char val) {
	Serial.print(val);
	comma();
}

void PrintValueComma(int val) {
	Serial.print(val);
	comma();
}

void PrintValueComma(unsigned long val)
{
	Serial.print(val);
	comma();
}

// Used to read floating point values from the serial port
float readFloatSerial() {
#define SERIALFLOATSIZE 10
	byte index = 0;
	byte timeout = 0;
	char data[SERIALFLOATSIZE] = "";
	
	do {
		if (Serial.available() == 0) {
			delay(10);
			timeout++;
		}
		else {
			data[index] = Serial.read();
			timeout = 0;
			index++;
		}
	}  
	while ((index == 0 || data[index-1] != ';') && (timeout < 5) && (index < sizeof(data)-1));
	data[index] = '\0';
	return atof(data);
}

void comma() {
	Serial.print(',');
}

/*void printInt(int data) {
	byte msb, lsb;
	
	msb = data >> 8;
	lsb = data & 0xff;
	
	binaryPort->print(msb, BYTE);
	binaryPort->print(lsb, BYTE);
}*/

/*void sendBinaryFloat(float data) {
	union binaryFloatType {
		byte floatByte[4];
		float floatVal;
	} binaryFloat;
	
	binaryFloat.floatVal = data;
	binaryPort->print(binaryFloat.floatByte[3], BYTE);
	binaryPort->print(binaryFloat.floatByte[2], BYTE);
	binaryPort->print(binaryFloat.floatByte[1], BYTE);
	binaryPort->print(binaryFloat.floatByte[0], BYTE);
}*/

/*void sendBinaryuslong(unsigned long data) {
	union binaryuslongType {
		byte uslongByte[4];
		unsigned long uslongVal;
	} binaryuslong;
	
	binaryuslong.uslongVal = data;
	binaryPort->print(binaryuslong.uslongByte[3], BYTE);
	binaryPort->print(binaryuslong.uslongByte[2], BYTE);
	binaryPort->print(binaryuslong.uslongByte[1], BYTE);
	binaryPort->print(binaryuslong.uslongByte[0], BYTE);
}*/

#ifdef BinaryWrite
void fastTelemetry(void)
{
	// **************************************************************
	// ***************** Fast Transfer Of Sensor Data ***************
	// **************************************************************
	// AeroQuad.h defines the output rate to be 10ms
	// Since writing to UART is done by hardware, unable to measure data rate directly
	// Through analysis:  115200 baud = 115200 bits/second = 14400 bytes/second
	// If float = 4 bytes, then 3600 floats/second
	// If 10 ms output rate, then 36 floats/10ms
	// Number of floats written using sendBinaryFloat is 15
	
	if (armed == ON) {
#ifdef OpenlogBinaryWrite
		printInt(21845); // Start word of 0x5555
		sendBinaryuslong(currentTime);
		//        printInt((int)flightMode);
		for (byte axis = ROLL; axis < LASTAXIS; axis++) sendBinaryFloat(gyro.getData(axis));
		for (byte axis = XAXIS; axis < LASTAXIS; axis++) sendBinaryFloat(accel.getData(axis));
		//        sendBinaryFloat(accel.accelOneG);
#ifdef HeadingMagHold
		//          sendBinaryFloat(compass.hdgX);
		//          sendBinaryFloat(compass.hdgY);
		sendBinaryFloat(compass.getRawData(XAXIS));
		sendBinaryFloat(compass.getRawData(YAXIS));
		sendBinaryFloat(compass.getRawData(ZAXIS));
#else
		sendBinaryFloat(0.0);
		sendBinaryFloat(0.0);
		//          sendBinaryFloat(0.0);
#endif
		//        for (byte axis = ROLL; axis < ZAXIS; axis++) sendBinaryFloat(flightAngle->getData(axis));
		printInt(32767); // Stop word of 0x7FFF
#else
		printInt(21845); // Start word of 0x5555
		for (byte axis = ROLL; axis < LASTAXIS; axis++) sendBinaryFloat(gyro.getData(axis));
		for (byte axis = XAXIS; axis < LASTAXIS; axis++) sendBinaryFloat(accel.getData(axis));
		for (byte axis = ROLL; axis < LASTAXIS; axis++)
#ifdef HeadingMagHold
			sendBinaryFloat(compass.getRawData(axis));
#else
		sendBinaryFloat(0);
#endif
		for (byte axis = ROLL; axis < LASTAXIS; axis++) sendBinaryFloat(flightAngle->getGyroUnbias(axis));
		for (byte axis = ROLL; axis < LASTAXIS; axis++) sendBinaryFloat(flightAngle->getData(axis));
		printInt(32767); // Stop word of 0x7FFF
#endif
	}
}
#endif    
