/***********************************************
 *	FILE:	State.cpp
 *
 *	AUTHORS:	-Guillermo Maroto
 *
 *           
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/
#include "State.h"

void State::initializeState()
{
	// Initialize sensors
	gyro.initialize();
	accel.initialize(); 
	
	// Calibrate sensors
	gyro.autoZero(); // defined in Gyro.h
	
	// Flight angle estimation
    flightAngle.initialize(1.0, 0.0);  // with no compass, DCM matrix initalizes to a heading of 0 degrees
}

void State::calibrateState()
{
	gyro.calibrate();
	accel.calibrate();
}

void State::measureState()
{	
	// *********************** Read Critical Sensors **********************
	gyro.measure();
	accel.measure();
	
	// ****************** Calculate Absolute Angle *****************
	

	flightAngle.calculate(gyro.getData(ROLL),  \
						   gyro.getData(PITCH),                      \
						   gyro.getData(YAW),                        \
						   accel.getData(XAXIS),                     \
						   accel.getData(YAXIS),                     \
						   accel.getData(ZAXIS),                     \
						   accel.getOneG(),                          \
						   0.0,                                      \
						   0.0);
}